/**
 *		Lightweight Slider
 *
 *		Designed by: E. Rogan Creswick
 *
 *		This component is an object used for selection over a range of values.
 *		(see JSlider)
 **/


import java.awt.*;
import java.awt.event.*;

public class Slider extends Component implements Adjustable
{

    /**
     * A constant that indicates a horizontal slider.
     */
    public static final int	HORIZONTAL = 0;

    /**
     * A constant that indicates a vertical slider.
     */
    public static final int	VERTICAL   = 1;

    /**
     * The Scrollbar's orientation--being either horizontal or vertical.
     * This value should be specified when the scrollbar is being
     * created.<BR>
     * orientation can be either : <code>VERTICAL</code> or
     * <code>HORIZONTAL</code> only.
     *
     * @serial
     * @see getOrientation()
     * @see setOrientation()
     */
    int orientation;

    /**
     * The value of the Scrollbar.
     * value should be either greater than <code>minimum</code>
     * or less that <code>maximum</code>
     *
     * @serial
     * @see getValue()
     * @see setValue()
     */
    int value = 50;

    /**
     * The value of held changes if the mouse is being held on the handle.
     * or if it is released, this boolean is used to determine some of the
     * repaint steps.
     *
     * @serial
     * @see processMouseEvent()
     * @see paint()
     */
    boolean held = false;

    Dimension size;
    ActionListener actionListener;     // Post action events to listeners

    /**
     * For action handling with the adjustmentListener.
     *
     */
    AdjustmentListener adjustmentListener;

    int numPts = 4;
    int xPts[]= {50, 55, 60, 55};
    int yPts[] = new int[numPts];

    //int value = 50; // center of handle is at value.
    int oldLocation = 50;

    Polygon handle;
    Rectangle guide;

    Graphics offgc;
    Image offscreen = null;
    Dimension d;

    public Slider(int orientation, int width, int height)
    {
	this.orientation = orientation;

	if(orientation == HORIZONTAL)
	    size = new Dimension(width,height);
	else if(orientation == VERTICAL)
	    size = new Dimension(height,width);
	makeHandle();
	makeGuide();
	enableEvents(AWTEvent.MOUSE_EVENT_MASK);
	enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
    }// end Slider()
    
    public void update(Graphics g)
    {
	Graphics offgc;
	Image offscreen = null;
	Dimension d = size();
	
	// create the offscreen buffer and associated Graphics
	offscreen = createImage(d.width, d.height);
	offgc = offscreen.getGraphics();
	// clear the exposed area
	offgc.setColor(getBackground());
	offgc.fillRect(0, 0, d.width, d.height);
	offgc.setColor(getForeground());
	// do normal redraw
	paint(offgc);
	// transfer offscreen to window
	g.drawImage(offscreen, 0, 0, this);
    }
    
    /**
     *	paint(Graphics g); paints the component with doublebuffering
     *
     */
    public void paint(Graphics g)
    {
	d = size();
	// create the offscreen buffer and associated Graphics
	offscreen = createImage(d.width, d.height);
	offgc = offscreen.getGraphics();

	offgc.setColor(OurColors.inner);
	Color temp = offgc.getColor();
	offgc.setColor(getBackground());
	offgc.fillRect(0,0,d.width, d.height);
	offgc.setColor(temp);

	paintGuide(offgc);

	if(held)
	    offgc.setColor(this.getForeground().darker());
	else if(!held)
	    offgc.setColor(this.getForeground());


	paintPolygon(offgc);

	g.drawImage(offscreen, 0, 0, null);
	offgc.dispose();
    }// end paint()


    private void paintPolygon(Graphics offgc)
    {
	if(orientation == HORIZONTAL)
	    {
		if(value >= 5 && value <= size.width -5)
		    {
			xPts[0] = value - 5;
			xPts[1] = value;
			xPts[2] = value + 5;
			xPts[3] = value;
			//handle.translate(oldLocation-value, 0);
		    }
	    }else if(orientation == VERTICAL)
		{
		    if(value >= 5 && value <= size.height -5)
			{
			    //handle.translate(0, value-oldLocation);
			}
		}
	handle = new Polygon(xPts, yPts, numPts);

	Color firstColor = offgc.getColor();
	offgc.fillPolygon(handle);
	offgc.setColor(firstColor.darker().darker().darker());
	offgc.drawPolygon(handle);
	offgc.setColor(firstColor);
    }// end paitPolygon()

    private void paintGuide(Graphics g)
    {
	Color base = g.getColor().darker();

	g.drawRect(guide.x, guide.y,guide.width,guide.height);

	g.setColor(base.brighter());
	g.drawLine(guide.x+1, guide.y+1, guide.x + guide.width -1, guide.y+1);
	g.drawLine(guide.x+1, guide.y+1, guide.x + 1, guide.y + guide.height -1);

	g.setColor(base.brighter().brighter());
	g.drawLine(guide.x+1, guide.y+guide.height+1, guide.x + guide.width+1, guide.y+guide.height+1);
	g.drawLine(guide.x+guide.width+1, guide.y+1, guide.x+guide.width+1, guide.y+guide.height+1);

	g.setColor(base.darker().darker());
    }// end paintGuide()

    private void makeHandle()
    {
	//xPts = ;
	yPts[0] = size.height/2;
	yPts[1] = size.height/2 - 7;
	yPts[2] = size.height/2;
	yPts[3] = size.height/2 + 7;
	handle = new Polygon(xPts, yPts, numPts);
    }// end drawHandle()

    private void makeGuide()
    {
	if(orientation == HORIZONTAL)
	    {
		guide = new Rectangle(10,size.height/2 - 3,size.width - 20,6);
	    }else if(orientation == VERTICAL)
		{
		    guide = new Rectangle(size.width/2 -3, 10, 6, size.height - 20);
		}

    }// end makeGuide()



    /**
     * Determines the orientation of this scroll bar.
     * @return	  the orientation of this scroll bar, either
     *		     <code>Scrollbar.HORIZONTAL</code> or
     *		     <code>Scrollbar.VERTICAL</code>.
     * @see	  java.awt.Scrollbar#setOrientation
     */
    public int getOrientation() {
	return orientation;
    }


    public Dimension getPreferredSize()
    {
	return size;
    }

    public Dimension getMinimumSize()
    {
	return size;
    }

    public Dimension getMaximumSize()
    {
	return size;
    }

    public boolean isLightweight()
    {
	return true;
    }

    /**
     * Gets the current value of this scroll bar.
     * @return	    the current value of this scroll bar.
     * @see	    java.awt.Scrollbar#getMinimum
     * @see	    java.awt.Scrollbar#getMaximum
     */
    public int getValue() {
	return value;
    }

    public int getMaximum()
    {
	return 0;
    }

    public int getMinimum()
    {
	return 0;
    }

    public int getBlockIncrement()
    {
	return 0;
    }

    public int getUnitIncrement()
    {
	return 0;
    }

    public int getVisibleAmount()
    {
	return 0;
    }

    public void setBlockIncrement(int b)
    {
    }

    public void setMaximum(int max)
    {
    }

    public void setMinimum(int min)
    {
    }

    public void setUnitIncrement(int u)
    {
    }

    public void setValue(int v)
    {
	value = v;
	repaint();
    }

    public void setVisibleAmount(int v)
    {
    }
    /**
     * Adds the specified action listener to receive action events
     * from this button.
     * @param listener the action listener
     */
    public void addActionListener(ActionListener listener)
    {
	actionListener = AWTEventMulticaster.add(actionListener, listener);
	//enableEvents(AWTEvent.MOUSE_EVENT_MASK);
	//enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
    }

    /**
     * Adds the specified adjustment listener to receive instances of
     * <code>AdjustmentEvent</code> from this scroll bar.
     * If l is null, no exception is thrown and no action is performed.
     *
     * @param	     l the adjustment listener.
     * @see	     java.awt.event.AdjustmentEvent
     * @see	     java.awt.event.AdjustmentListener
     * @see	     java.awt.Scrollbar#removeAdjustmentListener
     * @since	     JDK1.1
     */
    public synchronized void addAdjustmentListener(AdjustmentListener l)
    {

	if (l == null) {
	    return;
	}
	System.out.println("added adjustment listener");
	adjustmentListener = AWTEventMulticaster.add(adjustmentListener, l);
	enableEvents(AWTEvent.ADJUSTMENT_EVENT_MASK);
	//newEventsOnly = true;
    }

    /**
     * Removes the specified adjustment listener so that it no longer
     * receives instances of <code>AdjustmentEvent</code> from this scroll bar.
     * If l is null, no exception is thrown and no action is performed.
     *
     * @param		l    the adjustment listener.
     * @see		java.awt.event.AdjustmentEvent
     * @see		java.awt.event.AdjustmentListener
     * @see		java.awt.Scrollbar#addAdjustmentListener
     * @since		JDK1.1
     */
    public synchronized void removeAdjustmentListener(AdjustmentListener l)
    {
	if (l == null) {
	    return;
	}
	adjustmentListener = AWTEventMulticaster.remove(adjustmentListener, l);
    }

    /**
     * Processes events on this scroll bar. If the event is an
     * instance of <code>AdjustmentEvent</code>, it invokes the
     * <code>processAdjustmentEvent</code> method.
     * Otherwise, it invokes its superclass's
     * <code>processEvent</code> method.
     * @param	     e the event.
     * @see	     java.awt.event.AdjustmentEvent
     * @see	     java.awt.Scrollbar#processAdjustmentEvent
     * @since	     JDK1.1
     */
    protected void processEvent(AWTEvent e) {
	if (e instanceof AdjustmentEvent) {
	    processAdjustmentEvent((AdjustmentEvent)e);
	    return;
	}
	if (e instanceof MouseEvent) {
	    processMouseEvent((MouseEvent)e);
	    return;
	}

	super.processEvent(e);
    }

    protected void processMouseEvent(MouseEvent e)
    {
	//System.out.println("MouseEvent e="+e);
	switch(e.getID())
	    {
	    case MouseEvent.MOUSE_PRESSED:
		if(handle.contains(e.getPoint()))
		    {
			held = true;
			repaint();
		    }
		break;

	    case MouseEvent.MOUSE_RELEASED:
		if(held)
		    {
			held = false;
			repaint();
		    }
		break;

	    case MouseEvent.MOUSE_ENTERED:
		break;

	    case MouseEvent.MOUSE_EXITED:
		if(held)
		    {
			held = false;
			repaint();
		    }
		break;

	    case MouseEvent.MOUSE_DRAGGED:
		//System.out.println("mouse Dragged");
		oldLocation = value;
		if(orientation == HORIZONTAL)
		    {
			value = e.getX();
		    }else if(orientation == VERTICAL)
			{
			    value = e.getY();
			}
		processAdjustmentEvent(new AdjustmentEvent(this, AWTEvent.RESERVED_ID_MAX+1,1,value));
		repaint();
		break;

	    }// end switch.
	super.processMouseEvent(e);
    }// end processMouseEvent()
    /*
      protected void processMouseMotionEvent(MouseEvent e)
      {
      switch(e.getID())
      {
      case MouseEvent.MOUSE_DRAGGED:
      //System.out.println("mouse Dragged");
      oldLocation = value;
      if(orientation == HORIZONTAL)
      {
      value = e.getX();
      }else if(orientation == VERTICAL)
      {
      value = e.getY();
      }
      repaint();
      break;

      }// end switch.
      super.processMouseMotionEvent(e);
      }// end processMouseMotionEvent()
    */
	/**
	 * Processes adjustment events occurring on this
	 * scrollbar by dispatching them to any registered
	 * <code>AdjustmentListener</code> objects.
	 * <p>
	 * This method is not called unless adjustment events are
	 * enabled for this component. Adjustment events are enabled
	 * when one of the following occurs:
	 * <p><ul>
	 * <li>An <code>AdjustmentListener</code> object is registered
	 * via <code>addAdjustmentListener</code>.
	 * <li>Adjustment events are enabled via <code>enableEvents</code>.
	 * </ul><p>
	 * @param	e the adjustment event.
	 * @see 	java.awt.event.AdjustmentEvent
	 * @see 	java.awt.event.AdjustmentListener
	 * @see 	java.awt.Scrollbar#addAdjustmentListener
	 * @see 	java.awt.Component#enableEvents
	 * @since	JDK1.1
	 */
    protected void processAdjustmentEvent(AdjustmentEvent e) {
	if (adjustmentListener != null) {
	    adjustmentListener.adjustmentValueChanged(e);
	}
    }

    }// end class Slider()
